# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import os
from six.moves.urllib.parse import urljoin

import requests


API_URL = 'https://api.monzo.com/'
MONZO_ACCESS_TOKEN = 'MONZO_ACCESS_TOKEN'


class MonzoAPI(object):
    """
    Wrapper for Monzo public API

    Official docs:
        https://monzo.com/docs/
    """
    def __init__(self, access_token=None, default_account_id=None):
        # If no access token is provided, try to get it
        # from environment variable
        if not access_token and os.environ.get(MONZO_ACCESS_TOKEN):
            access_token = os.environ.get(MONZO_ACCESS_TOKEN)
        elif not access_token and not os.environ.get(MONZO_ACCESS_TOKEN):
            raise ValueError("No access token provided")
        self._access_token = access_token

        # Init requests session and add auth header
        self.session = requests.Session()
        self.session.headers.update({
            'Authorization': 'Bearer {0}'.format(self._access_token),
        })

        # Make sure that the access token is correct
        if ('authenticated' not in self.whoami() or
                not self.whoami()['authenticated']):
            raise ValueError("Incorrect access token")

        # Set the account id if it wasn't provided
        # and there's only one available
        if not default_account_id and len(self.accounts()) == 1:
            self.default_account_id = self.accounts()[0]['id']
        else:
            self.default_account_id = default_account_id

    def whoami(self):
        """
        Get information about an access token

        Official docs:
            https://monzo.com/docs/#authenticating-requests
        """
        endpoint = '/ping/whoami'
        url = urljoin(API_URL, endpoint)
        response = self.session.get(url)

        return response.json()

    def accounts(self):
        """
        Returns a list of accounts owned by the currently authorised user.

        Official docs:
            https://monzo.com/docs/#list-accounts
        """
        endpoint = '/accounts'
        url = urljoin(API_URL, endpoint)
        response = self.session.get(url)

        return response.json()['accounts']

    def balance(self, account_id=None):
        """
        Returns balance information for a specific account.

        Official docs:
            https://monzo.com/docs/#read-balance
        """
        if not account_id and not self.default_account_id:
            raise ValueError("You need to pass the account ID")
        elif not account_id and self.default_account_id:
            account_id = self.default_account_id

        endpoint = '/balance'
        url = urljoin(API_URL, endpoint)
        data = {
            'account_id': account_id,
        }
        response = self.session.get(url, params=data)

        return response.json()

    def transactions(self, account_id=None, reverse=True, limit=None):
        """
        Returns a list of transactions on the user's account.

        Official docs:
            https://monzo.com/docs/#list-transactions
        """
        if not account_id and not self.default_account_id:
            raise ValueError("You need to pass the account ID")
        elif not account_id and self.default_account_id:
            account_id = self.default_account_id

        endpoint = '/transactions'
        url = urljoin(API_URL, endpoint)
        data = {
            'account_id': account_id,
        }
        response = self.session.get(url, params=data)

        # The API does not allow reversing the list and getting the latest
        # transactions first. To allow a basic query of 'the latest transaction'
        # we need to always get all transactions (i.e. not use the 'limit'
        # parameter) and do the reversing and slicing in Python
        # I send Monzo an email, we'll se how they'll respond
        transactions = response.json()['transactions']
        if reverse:
            transactions.reverse()

        if limit:
            return transactions[:limit]
        else:
            return transactions

    def transaction(self, transaction_id, expand_merchant=False):
        """
        Returns an individual transaction, fetched by its id.

        Official docs:
            https://monzo.com/docs/#retrieve-transaction
        """
        endpoint = '/transactions/{}'.format(transaction_id)
        url = urljoin(API_URL, endpoint)

        data = dict()
        if expand_merchant:
            data.update({'expand[]': 'merchant' if expand_merchant else ''})

        response = self.session.get(url, params=data)

        return response.json()['transaction']



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import os
from uuid import uuid4

import pytest

from pymonzo import MonzoAPI
from pymonzo.monzo_api import MONZO_ACCESS_TOKEN


# Fixtures
@pytest.fixture(scope='function')
def monzo_api():
    """Create a MonzoAPI instance"""
    return MonzoAPI()


# Tests
def test_init_access_token(monkeypatch):
    """Test access token initializing in MonzoAPI"""
    # Get access token from environment variable
    assert MonzoAPI(access_token=None)

    # Pass access token explicitly
    access_token = os.environ['MONZO_ACCESS_TOKEN']
    monzo_api = MonzoAPI(access_token=access_token)
    assert monzo_api
    assert monzo_api._access_token == access_token

    # Incorrect access token
    with pytest.raises(ValueError):
        monkeypatch.undo()
        monkeypatch.delenv(MONZO_ACCESS_TOKEN, raising=False)
        MonzoAPI(access_token=str(uuid4()))

    # Incorrect access token from environment variable
    with pytest.raises(ValueError):
        monkeypatch.undo()
        monkeypatch.setenv(MONZO_ACCESS_TOKEN, str(uuid4()))
        MonzoAPI()

    # No access token
    with pytest.raises(ValueError):
        monkeypatch.undo()
        monkeypatch.delenv(MONZO_ACCESS_TOKEN, raising=False)
        MonzoAPI()


def test_init_session(monzo_api):
    """Test default account ID initializing in MonzoAPI"""
    assert monzo_api.session
    assert 'Authorization' in monzo_api.session.headers


def test_init_default_account_id(monzo_api):
    """Test default account ID initializing in MonzoAPI"""
    assert monzo_api.default_account_id == monzo_api.accounts()[0]['id']

    # Passing default account ID explicitly
    account_id = monzo_api.accounts()[0]['id']
    monzo_api = MonzoAPI(default_account_id=account_id)
    assert monzo_api.default_account_id == account_id


def test_whoami(monzo_api):
    """Test MonzoAPI.whoami() method"""
    assert monzo_api.whoami()


def test_accounts(monzo_api):
    """Test MonzoAPI.accounts() method"""
    assert monzo_api.accounts()


def test_balance(monzo_api):
    """Test MonzoAPI.balance() method"""
    assert monzo_api.balance()

    # No account ID provided
    with pytest.raises(ValueError):
        monzo_api.default_account_id = None
        monzo_api.balance()


def test_transactions(monzo_api):
    """Test MonzoAPI.transactions() method"""
    transactions = monzo_api.transactions()
    assert transactions

    # Limit results
    assert len(monzo_api.transactions(limit=5)) == 5

    # No real way to test reverse, but make sure it does _something_
    transactions_reverse = monzo_api.transactions(reverse=True)
    assert transactions is not transactions_reverse

    # No account ID provided
    with pytest.raises(ValueError):
        monzo_api.default_account_id = None
        monzo_api.transactions()


def test_transaction(monzo_api):
    """Test MonzoAPI.transaction() method"""
    transaction_id = monzo_api.transactions(limit=1)[0]['id']

    transaction = monzo_api.transaction(transaction_id=transaction_id)
    assert transaction

    transaction_expand_merchant = monzo_api.transaction(
        transaction_id=transaction_id, expand_merchant=True,
    )
    assert transaction_expand_merchant

    # No real way to test reverse, but make sure it does _something_
    assert transaction is not transaction_expand_merchant



# -*- coding: utf-8 -*-
from __future__ import print_function

import io
import os
import re

from setuptools import setup, find_packages


# Convert description from markdown to reStructuredText
try:
    import pypandoc
    description = pypandoc.convert('README.md', 'rst', 'markdown')
except (OSError, ImportError):
    description = ''


# Get package version number
# Source: https://packaging.python.org/single_source_version/
def read(*names, **kwargs):
    with io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ) as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(
        r"^__version__ = ['\"]([^'\"]*)['\"]",
        version_file, re.M
    )
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='pymonzo',
    url='https://github.com/pawelad/pymonzo',
    download_url='https://github.com/pawelad/pymonzo/releases/latest',
    bugtrack_url='https://github.com/pawelad/pymonzo/issues',
    version=find_version('pymonzo', '__init__.py'),
    license='MIT License',
    author='Paweł Adamczak',
    author_email='pawel.ad@gmail.com',
    maintainer='Paweł Adamczak',
    maintainer_email='pawel.ad@gmail.com',
    description='Python wrapper for Mondo public API',
    long_description=description,
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'requests>=2.11.1',
        'six>=1.10.0',
    ],
    extras_require={
        'testing': ['pytest'],
    },
    keywords='icon font export font awesome octicons',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
